@@ -876,6 +876,10 @@ static int32_t ctdb_control_dispatch(struct ctdb_context *ctdb,
 		CHECK_CONTROL_DATA_SIZE(sizeof(struct ctdb_connection));
 		return ctdb_control_tcp_client_passed(ctdb, client_id, indata);
 
+	case CTDB_CONTROL_START_IPREALLOCATE:
+		CHECK_CONTROL_DATA_SIZE(0);
+		return ctdb_control_start_ipreallocate(ctdb, c, async_reply);
+
 	default:
 		DEBUG(DEBUG_CRIT,(__location__ " Unknown CTDB control opcode %u\n", opcode));
 		return -1;
