@@ -44,7 +44,7 @@
 #include "common/common.h"
 #include "common/logging.h"
 
-#include "server/ctdb_config.h"
+#include "conf/ctdb_config.h"
 
 #include "ctdb_cluster_mutex.h"
 
@@ -395,7 +395,7 @@ static void ctdb_set_culprit_count(struct ctdb_recoverd *rec,
 				   struct ctdb_banning_state,
 				   len);
 		if (t == NULL) {
-			DBG_WARNING("Memory allocation error");
+			DBG_WARNING("Memory allocation error\n");
 			return;
 		}
 		rec->banning_state = t;
@@ -1311,13 +1311,13 @@ static int do_recovery(struct ctdb_recoverd *rec, TALLOC_CTX *mem_ctx)
 	int ret;
 	bool self_ban;
 
-	DEBUG(DEBUG_NOTICE, (__location__ " Starting do_recovery\n"));
+	DBG_NOTICE("Starting do_recovery\n");
 
 	/* Check if the current node is still the leader.  It's possible that
 	 * re-election has changed the leader.
 	 */
 	if (!this_node_is_leader(rec)) {
-		D_NOTICE("Leader changed to %u, aborting recovery\n",
+		D_NOTICE("Leader changed to %" PRIu32 ", aborting recovery\n",
 			 rec->leader);
 		return -1;
 	}
@@ -1331,20 +1331,22 @@ static int do_recovery(struct ctdb_recoverd *rec, TALLOC_CTX *mem_ctx)
 
 	if (rec->election_in_progress) {
 		/* an election is in progress */
-		DEBUG(DEBUG_ERR, ("do_recovery called while election in progress - try again later\n"));
+		DBG_ERR("do_recovery called while election in progress - try "
+			"again later\n");
 		goto fail;
 	}
 
 	ban_misbehaving_nodes(rec, &self_ban);
 	if (self_ban) {
-		DEBUG(DEBUG_NOTICE, ("This node was banned, aborting recovery\n"));
+		DBG_NOTICE("This node was banned, aborting recovery\n");
 		goto fail;
 	}
 
 	if (cluster_lock_enabled(rec) && !cluster_lock_held(rec)) {
 		/* Leader can change in ban_misbehaving_nodes() */
 		if (!this_node_is_leader(rec)) {
-			D_NOTICE("Leader changed to %u, aborting recovery\n",
+			D_NOTICE("Leader changed to %" PRIu32
+				 ", aborting recovery\n",
 				 rec->leader);
 			rec->need_recovery = false;
 			goto fail;
@@ -1355,12 +1357,13 @@ static int do_recovery(struct ctdb_recoverd *rec, TALLOC_CTX *mem_ctx)
 		goto fail;
 	}
 
-	DEBUG(DEBUG_NOTICE, (__location__ " Recovery initiated due to problem with node %u\n", rec->last_culprit_node));
+	DBG_NOTICE("Recovery initiated due to problem with node %" PRIu32 "\n",
+		   rec->last_culprit_node);
 
 	/* Retrieve capabilities from all connected nodes */
 	ret = update_capabilities(rec, nodemap);
 	if (ret!=0) {
-		DEBUG(DEBUG_ERR, (__location__ " Unable to update node capabilities.\n"));
+		DBG_ERR("Unable to update node capabilities.\n");
 		return -1;
 	}
 
@@ -1377,15 +1380,19 @@ static int do_recovery(struct ctdb_recoverd *rec, TALLOC_CTX *mem_ctx)
 						nodemap->nodes[i].flags);
 		if (ret != 0) {
 			if (nodemap->nodes[i].flags & NODE_FLAGS_INACTIVE) {
-				DEBUG(DEBUG_WARNING, (__location__ "Unable to update flags on inactive node %d\n", i));
+				DBG_WARNING("Unable to update flags on "
+					    "inactive node %d\n",
+					    i);
 			} else {
-				DEBUG(DEBUG_ERR, (__location__ " Unable to update flags on all nodes for node %d\n", i));
+				DBG_ERR("Unable to update flags on all nodes "
+					"for node %d\n",
+					i);
 				return -1;
 			}
 		}
 	}
 
-	DEBUG(DEBUG_NOTICE, (__location__ " Recovery - updated flags\n"));
+	DBG_NOTICE("Recovery - updated flags\n");
 
 	ret = db_recovery_parallel(rec, mem_ctx);
 	if (ret != 0) {
@@ -1399,11 +1406,11 @@ static int do_recovery(struct ctdb_recoverd *rec, TALLOC_CTX *mem_ctx)
 	ret = ctdb_client_send_message(ctdb, CTDB_BROADCAST_CONNECTED,
 				       CTDB_SRVID_RECONFIGURE, tdb_null);
 	if (ret != 0) {
-		DEBUG(DEBUG_ERR, (__location__ " Failed to send reconfigure message\n"));
+		DBG_ERR("Failed to send reconfigure message\n");
 		goto fail;
 	}
 
-	DEBUG(DEBUG_NOTICE, (__location__ " Recovery complete\n"));
+	DBG_NOTICE("Recovery complete\n");
 
 	rec->need_recovery = false;
 	ctdb_op_end(rec->recovery);
@@ -1417,7 +1424,10 @@ static int do_recovery(struct ctdb_recoverd *rec, TALLOC_CTX *mem_ctx)
 	   We now wait for rerecovery_timeout before we allow
 	   another recovery to take place.
 	*/
-	DEBUG(DEBUG_NOTICE, ("Just finished a recovery. New recoveries will now be suppressed for the rerecovery timeout (%d seconds)\n", ctdb->tunable.rerecovery_timeout));
+	D_NOTICE("Just finished a recovery. New recoveries will now be "
+		 "suppressed for the rerecovery timeout (%" PRIu32
+		 " seconds)\n",
+		 ctdb->tunable.rerecovery_timeout);
 	ctdb_op_disable(rec->recovery, ctdb->ev,
 			ctdb->tunable.rerecovery_timeout);
 	return 0;
@@ -1614,7 +1624,7 @@ static void reload_nodes_handler(uint64_t srvid, TDB_DATA data,
 
 	DEBUG(DEBUG_ERR, (__location__ " Reload nodes file from recovery daemon\n"));
 
-	ctdb_load_nodes_file(rec->ctdb);
+	ctdb_load_nodes(rec->ctdb);
 }
 
 
@@ -1892,6 +1902,7 @@ static void cluster_lock_election(struct ctdb_recoverd *rec)
 	 * attempt to retake it.  This provides stability.
 	 */
 	if (cluster_lock_held(rec)) {
+		rec->leader = rec->pnn;
 		goto done;
 	}
 
@@ -1943,7 +1954,7 @@ static void force_election(struct ctdb_recoverd *rec)
 
 	ret = send_election_request(rec);
 	if (ret!=0) {
-		DBG_ERR("Failed to initiate leader election");
+		DBG_ERR("Failed to initiate leader election\n");
 		return;
 	}
 
@@ -2275,6 +2286,7 @@ static int verify_local_ip_allocation(struct ctdb_recoverd *rec)
 	int ret;
 	bool need_takeover_run = false;
 	struct ctdb_public_ip_list_old *ips = NULL;
+	struct ctdb_sys_local_ips_context *ips_ctx = NULL;
 
 	/* If we are not the leader then do some housekeeping */
 	if (!this_node_is_leader(rec)) {
@@ -2345,24 +2357,48 @@ static int verify_local_ip_allocation(struct ctdb_recoverd *rec)
 		return -1;
 	}
 
+	ret = ctdb_sys_local_ips_init(mem_ctx, &ips_ctx);
+	if (ret != 0) {
+		/*
+		 * What to do?  The point here is to allow public
+		 * addresses to be checked when
+		 * net.ipv4.ip_nonlocal_bind = 1, which is probably
+		 * just Linux... though other platforms may have a
+		 * similar setting.  For non-Linux platforms without a
+		 * usable getifaddrs(3) function/replacement, fall
+		 * back to bind() below...
+		 */
+		DBG_DEBUG("Failed to get local addresses, depending on bind\n");
+		ips_ctx = NULL; /* Just in case */
+	}
+
 	for (j=0; j<ips->num; j++) {
+		ctdb_sock_addr *addr = &ips->ips[j].addr;
+		bool have_ip;
+
+		if (ips_ctx != NULL) {
+			have_ip = ctdb_sys_local_ip_check(ips_ctx, addr);
+		} else {
+			have_ip = ctdb_sys_bind_ip_check(addr);
+		}
+
 		if (ips->ips[j].pnn == rec->pnn) {
-			if (!ctdb_sys_have_ip(&ips->ips[j].addr)) {
-				DEBUG(DEBUG_ERR,
-				      ("Assigned IP %s not on an interface\n",
-				       ctdb_addr_to_str(&ips->ips[j].addr)));
+			if (!have_ip) {
+				D_ERR("Assigned IP %s not on an interface\n",
+				      ctdb_addr_to_str(addr));
 				need_takeover_run = true;
 			}
 		} else {
-			if (ctdb_sys_have_ip(&ips->ips[j].addr)) {
-				DEBUG(DEBUG_ERR,
-				      ("IP %s incorrectly on an interface\n",
-				       ctdb_addr_to_str(&ips->ips[j].addr)));
+			if (have_ip) {
+				D_ERR("IP %s incorrectly on an interface\n",
+				      ctdb_addr_to_str(addr));
 				need_takeover_run = true;
 			}
 		}
 	}
 
+	TALLOC_FREE(ips_ctx);
+
 done:
 	if (need_takeover_run) {
 		struct ctdb_srvid_message rd;
@@ -2667,7 +2703,7 @@ static void main_loop(struct ctdb_context *ctdb, struct ctdb_recoverd *rec,
 
 	if (ctdb->num_nodes != nodemap->num) {
 		DEBUG(DEBUG_ERR, (__location__ " ctdb->num_nodes (%d) != nodemap->num (%d) reloading nodes file\n", ctdb->num_nodes, nodemap->num));
-		ctdb_load_nodes_file(ctdb);
+		ctdb_load_nodes(ctdb);
 		return;
 	}
 
