@@ -15,29 +15,32 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-
-from __future__ import print_function
 """NT Acls."""
 
 
 import os
-import tarfile
 import tempfile
 import shutil
 
-import samba.xattr_native, samba.xattr_tdb, samba.posix_eadb
+import samba.xattr_native
+import samba.xattr_tdb
+import samba.posix_eadb
 from samba.samba3 import param as s3param
 from samba.dcerpc import security, xattr, idmap
 from samba.ndr import ndr_pack, ndr_unpack
 from samba.samba3 import smbd
-from samba.auth import admin_session
-from samba import smb
+from samba.samba3 import libsmb_samba_internal as libsmb
+from samba.logger import get_samba_logger
+from samba import NTSTATUSError
+from samba.auth_util import system_session_unix
+from samba import safe_tarfile as tarfile
+
 
 # don't include volumes
-SMB_FILE_ATTRIBUTE_FLAGS = smb.FILE_ATTRIBUTE_SYSTEM | \
-                           smb.FILE_ATTRIBUTE_DIRECTORY | \
-                           smb.FILE_ATTRIBUTE_ARCHIVE | \
-                           smb.FILE_ATTRIBUTE_HIDDEN
+SMB_FILE_ATTRIBUTE_FLAGS = libsmb.FILE_ATTRIBUTE_SYSTEM | \
+                           libsmb.FILE_ATTRIBUTE_DIRECTORY | \
+                           libsmb.FILE_ATTRIBUTE_ARCHIVE | \
+                           libsmb.FILE_ATTRIBUTE_HIDDEN
 
 
 SECURITY_SECINFO_FLAGS = security.SECINFO_OWNER | \
@@ -45,18 +48,12 @@ SECURITY_SECINFO_FLAGS = security.SECINFO_OWNER | \
                          security.SECINFO_DACL  | \
                          security.SECINFO_SACL
 
-
-# SEC_FLAG_SYSTEM_SECURITY is required otherwise get Access Denied
-SECURITY_SEC_FLAGS = security.SEC_FLAG_SYSTEM_SECURITY | \
-                     security.SEC_FLAG_MAXIMUM_ALLOWED
-
-
 class XattrBackendError(Exception):
     """A generic xattr backend error."""
 
 
 def checkset_backend(lp, backend, eadbfile):
-    '''return the path to the eadb, or None'''
+    """return the path to the eadb, or None"""
     if backend is None:
         xattr_tdb = lp.get("xattr_tdb:file")
         if xattr_tdb is not None:
@@ -76,9 +73,12 @@ def checkset_backend(lp, backend, eadbfile):
         if eadbfile is not None:
             return (samba.xattr_tdb, eadbfile)
         else:
-            return (samba.xattr_tdb, os.path.abspath(os.path.join(lp.get("state dir"), "xattr.tdb")))
+            state_dir = lp.get("state directory")
+            db_path = os.path.abspath(os.path.join(state_dir, "xattr.tdb"))
+            return (samba.xattr_tdb, db_path)
     else:
-        raise XattrBackendError("Invalid xattr backend choice %s"%backend)
+        raise XattrBackendError("Invalid xattr backend choice %s" % backend)
+
 
 def getdosinfo(lp, file):
     try:
@@ -89,7 +89,14 @@ def getdosinfo(lp, file):
 
     return ndr_unpack(xattr.DOSATTRIB, attribute)
 
-def getntacl(lp, file, backend=None, eadbfile=None, direct_db_access=True, service=None):
+
+def getntacl(lp,
+             file,
+             session_info,
+             backend=None,
+             eadbfile=None,
+             direct_db_access=True,
+             service=None):
     if direct_db_access:
         (backend_obj, dbname) = checkset_backend(lp, backend, eadbfile)
         if dbname is not None:
@@ -115,13 +122,16 @@ def getntacl(lp, file, backend=None, eadbfile=None, direct_db_access=True, servi
         elif ntacl.version == 4:
             return ntacl.info.sd
     else:
-        return smbd.get_nt_acl(file, SECURITY_SECINFO_FLAGS, service=service)
+        return smbd.get_nt_acl(file,
+                               SECURITY_SECINFO_FLAGS,
+                               session_info,
+                               service=service)
 
 
-def setntacl(lp, file, sddl, domsid,
+def setntacl(lp, file, sddl, domsid, session_info,
              backend=None, eadbfile=None,
              use_ntvfs=True, skip_invalid_chown=False,
-             passdb=None, service=None, session_info=None):
+             passdb=None, service=None):
     """
     A wrapper for smbd set_nt_acl api.
 
@@ -174,7 +184,8 @@ def setntacl(lp, file, sddl, domsid,
 
                     smbd.set_nt_acl(
                         file, SECURITY_SECINFO_FLAGS, sd2,
-                        service=service, session_info=session_info)
+                        session_info,
+                        service=service)
 
                     # and then set an NTVFS ACL (which does not set the posix ACL) to pretend the owner really was set
                     use_ntvfs = True
@@ -192,7 +203,9 @@ def setntacl(lp, file, sddl, domsid,
                     security.SECINFO_GROUP |
                     security.SECINFO_DACL |
                     security.SECINFO_SACL,
-                    sd, service=service, session_info=session_info)
+                    sd,
+                    session_info,
+                    service=service)
 
     if use_ntvfs:
         (backend_obj, dbname) = checkset_backend(lp, backend, eadbfile)
@@ -255,14 +268,14 @@ def ldapmask2filemask(ldm):
 
     if (ldm & RIGHT_DS_READ_PROPERTY) and (ldm & RIGHT_DS_LIST_CONTENTS):
         filemask = filemask | (SYNCHRONIZE | FILE_LIST_DIRECTORY |
-                                FILE_READ_ATTRIBUTES | FILE_READ_EA |
-                                FILE_READ_DATA | FILE_EXECUTE)
+                               FILE_READ_ATTRIBUTES | FILE_READ_EA |
+                               FILE_READ_DATA | FILE_EXECUTE)
 
     if ldm & RIGHT_DS_WRITE_PROPERTY:
         filemask = filemask | (SYNCHRONIZE | FILE_WRITE_DATA |
-                                FILE_APPEND_DATA | FILE_WRITE_EA |
-                                FILE_WRITE_ATTRIBUTES | FILE_ADD_FILE |
-                                FILE_ADD_SUBDIRECTORY)
+                               FILE_APPEND_DATA | FILE_WRITE_EA |
+                               FILE_WRITE_ATTRIBUTES | FILE_ADD_FILE |
+                               FILE_ADD_SUBDIRECTORY)
 
     if ldm & RIGHT_DS_CREATE_CHILD:
         filemask = filemask | (FILE_ADD_SUBDIRECTORY | FILE_ADD_FILE)
@@ -286,17 +299,32 @@ def dsacl2fsacl(dssddl, sid, as_sddl=True):
     fdescr.group_sid = ref.group_sid
     fdescr.type = ref.type
     fdescr.revision = ref.revision
-    aces = ref.dacl.aces
-    for i in range(0, len(aces)):
-        ace = aces[i]
-        if not ace.type & security.SEC_ACE_TYPE_ACCESS_ALLOWED_OBJECT and str(ace.trustee) != security.SID_BUILTIN_PREW2K:
-       #    if fdescr.type & security.SEC_DESC_DACL_AUTO_INHERITED:
-            ace.flags = ace.flags | security.SEC_ACE_FLAG_OBJECT_INHERIT | security.SEC_ACE_FLAG_CONTAINER_INHERIT
-            if str(ace.trustee) == security.SID_CREATOR_OWNER:
-                # For Creator/Owner the IO flag is set as this ACE has only a sense for child objects
-                ace.flags = ace.flags | security.SEC_ACE_FLAG_INHERIT_ONLY
-            ace.access_mask =  ldapmask2filemask(ace.access_mask)
-            fdescr.dacl_add(ace)
+
+    # Only apply allowed and deny ACEs, as they are the only ones
+    # we can map to filesystem aces.
+    #
+    # In future we may need to include resource based aces...
+    allowed_ace_types = [
+        security.SEC_ACE_TYPE_ACCESS_ALLOWED,
+        security.SEC_ACE_TYPE_ACCESS_DENIED,
+    ]
+
+    for ace in ref.dacl.aces:
+        if not ace.type in allowed_ace_types:
+            continue
+
+        # Don't add the allow for SID_BUILTIN_PREW2K as in
+        # gp_create_gpt_security_descriptor()
+        if str(ace.trustee) == security.SID_BUILTIN_PREW2K:
+            continue
+
+        ace.flags = ace.flags | security.SEC_ACE_FLAG_OBJECT_INHERIT | security.SEC_ACE_FLAG_CONTAINER_INHERIT
+        if str(ace.trustee) == security.SID_CREATOR_OWNER:
+            # For Creator/Owner the IO flag is set as this ACE has only a sense for child objects
+            ace.flags = ace.flags | security.SEC_ACE_FLAG_INHERIT_ONLY
+
+        ace.access_mask = ldapmask2filemask(ace.access_mask)
+        fdescr.dacl_add(ace)
 
     if not as_sddl:
         return fdescr
@@ -315,14 +343,30 @@ class SMBHelper:
         self.smb_conn = smb_conn
         self.dom_sid = dom_sid
 
-    def get_acl(self, smb_path, as_sddl=False):
+    def get_acl(self, smb_path, as_sddl=False,
+                sinfo=None, access_mask=None):
         assert '/' not in smb_path
 
-        ntacl_sd = self.smb_conn.get_acl(
-            smb_path, SECURITY_SECINFO_FLAGS, SECURITY_SEC_FLAGS)
+        ntacl_sd = self.smb_conn.get_acl(smb_path,
+                                         sinfo=sinfo,
+                                         access_mask=access_mask)
 
         return ntacl_sd.as_sddl(self.dom_sid) if as_sddl else ntacl_sd
 
+    def set_acl(self, smb_path, ntacl_sd,
+                sinfo=None, access_mask=None):
+        assert '/' not in smb_path
+
+        assert(isinstance(ntacl_sd, str) or isinstance(ntacl_sd, security.descriptor))
+        if isinstance(ntacl_sd, str):
+            tmp_desc = security.descriptor.from_sddl(ntacl_sd, self.domain_sid)
+        elif isinstance(ntacl_sd, security.descriptor):
+            tmp_desc = ntacl_sd
+
+        self.smb_conn.set_acl(smb_path, tmp_desc,
+                              sinfo=sinfo,
+                              access_mask=access_mask)
+
     def list(self, smb_path=''):
         """
         List file and dir base names in smb_path without recursive.
@@ -336,7 +380,7 @@ class SMBHelper:
 
         attrib is from list method.
         """
-        return bool(attrib & smb.FILE_ATTRIBUTE_DIRECTORY)
+        return bool(attrib & libsmb.FILE_ATTRIBUTE_DIRECTORY)
 
     def join(self, root, name):
         """
@@ -429,20 +473,21 @@ class NtaclsHelper:
 
         self.use_ntvfs = "smb" in self.lp.get("server services")
 
-    def getntacl(self, path, as_sddl=False, direct_db_access=None):
+    def getntacl(self, path, session_info, as_sddl=False, direct_db_access=None):
         if direct_db_access is None:
             direct_db_access = self.use_ntvfs
 
         ntacl_sd = getntacl(
-            self.lp, path,
+            self.lp, path, session_info,
             direct_db_access=direct_db_access,
             service=self.service)
 
         return ntacl_sd.as_sddl(self.dom_sid) if as_sddl else ntacl_sd
 
-    def setntacl(self, path, ntacl_sd):
+    def setntacl(self, path, ntacl_sd, session_info):
         # ntacl_sd can be obj or str
-        return setntacl(self.lp, path, ntacl_sd, self.dom_sid)
+        return setntacl(self.lp, path, ntacl_sd, self.dom_sid, session_info,
+                        use_ntvfs=self.use_ntvfs)
 
 
 def _create_ntacl_file(dst, ntacl_sddl_str):
@@ -451,21 +496,28 @@ def _create_ntacl_file(dst, ntacl_sddl_str):
 
 
 def _read_ntacl_file(src):
-    with open(src + '.NTACL', 'r') as f:
+    ntacl_file = src + '.NTACL'
+
+    if not os.path.exists(ntacl_file):
+        return None
+
+    with open(ntacl_file, 'r') as f:
         return f.read()
 
 
 def backup_online(smb_conn, dest_tarfile_path, dom_sid):
     """
-    Backup all files and dirs with ntacl for the serive behind smb_conn.
+    Backup all files and dirs with ntacl for the service behind smb_conn.
 
     1. Create a temp dir as container dir
     2. Backup all files with dir structure into container dir
-    3. Generate file.NTACL files for each file and dir in contianer dir
+    3. Generate file.NTACL files for each file and dir in container dir
     4. Create a tar file from container dir(without top level folder)
-    5. Delete contianer dir
+    5. Delete container dir
     """
 
+    logger = get_samba_logger()
+
     if isinstance(dom_sid, str):
         dom_sid = security.dom_sid(dom_sid)
 
@@ -496,8 +548,14 @@ def backup_online(smb_conn, dest_tarfile_path, dom_sid):
                     f.write(data)
 
             # get ntacl for this entry and save alongside
-            ntacl_sddl_str = smb_helper.get_acl(r_name, as_sddl=True)
-            _create_ntacl_file(l_name, ntacl_sddl_str)
+            try:
+                ntacl_sddl_str = smb_helper.get_acl(r_name, as_sddl=True)
+                _create_ntacl_file(l_name, ntacl_sddl_str)
+            except NTSTATUSError as e:
+                logger.error('Failed to get the ntacl for %s: %s' %
+                             (r_name, e.args[1]))
+                logger.warning('The permissions for %s may not be' % r_name +
+                               ' restored correctly')
 
     with tarfile.open(name=dest_tarfile_path, mode='w:gz') as tar:
         for name in os.listdir(localdir):
@@ -507,15 +565,13 @@ def backup_online(smb_conn, dest_tarfile_path, dom_sid):
     shutil.rmtree(localdir)
 
 
-def backup_offline(src_service_path, dest_tarfile_path, samdb_conn, smb_conf_path):
+def backup_offline(src_service_path, dest_tarfile_path, smb_conf_path, dom_sid):
     """
     Backup files and ntacls to a tarfile for a service
     """
     service = src_service_path.rstrip('/').rsplit('/', 1)[-1]
     tempdir = tempfile.mkdtemp()
-
-    dom_sid_str = samdb_conn.get_domain_sid()
-    dom_sid = security.dom_sid(dom_sid_str)
+    session_info = system_session_unix()
 
     ntacls_helper = NtaclsHelper(service, smb_conf_path, dom_sid)
 
@@ -529,8 +585,8 @@ def backup_offline(src_service_path, dest_tarfile_path, samdb_conn, smb_conf_pat
             src = os.path.join(dirpath, dirname)
             dst = os.path.join(dst_dirpath, dirname)
             # mkdir with metadata
-            smbd.mkdir(dst, service)
-            ntacl_sddl_str = ntacls_helper.getntacl(src, as_sddl=True)
+            smbd.mkdir(dst, session_info, service)
+            ntacl_sddl_str = ntacls_helper.getntacl(src, session_info, as_sddl=True)
             _create_ntacl_file(dst, ntacl_sddl_str)
 
         # create files and NTACL file, then copy data
@@ -538,8 +594,8 @@ def backup_offline(src_service_path, dest_tarfile_path, samdb_conn, smb_conf_pat
             src = os.path.join(dirpath, filename)
             dst = os.path.join(dst_dirpath, filename)
             # create an empty file with metadata
-            smbd.create_file(dst, service)
-            ntacl_sddl_str = ntacls_helper.getntacl(src, as_sddl=True)
+            smbd.create_file(dst, session_info, service)
+            ntacl_sddl_str = ntacls_helper.getntacl(src, session_info, as_sddl=True)
             _create_ntacl_file(dst, ntacl_sddl_str)
 
             # now put data in
@@ -561,6 +617,7 @@ def backup_restore(src_tarfile_path, dst_service_path, samdb_conn, smb_conf_path
     """
     Restore files and ntacls from a tarfile to a service
     """
+    logger = get_samba_logger()
     service = dst_service_path.rstrip('/').rsplit('/', 1)[-1]
     tempdir = tempfile.mkdtemp()  # src files
 
@@ -568,6 +625,7 @@ def backup_restore(src_tarfile_path, dst_service_path, samdb_conn, smb_conf_path
     dom_sid = security.dom_sid(dom_sid_str)
 
     ntacls_helper = NtaclsHelper(service, smb_conf_path, dom_sid)
+    session_info = system_session_unix()
 
     with tarfile.open(src_tarfile_path) as f:
         f.extractall(path=tempdir)
@@ -584,9 +642,15 @@ def backup_restore(src_tarfile_path, dst_service_path, samdb_conn, smb_conf_path
                 dst = os.path.join(dst_dirpath, dirname)
                 if not os.path.isdir(dst):
                     # dst must be absolute path for smbd API
-                    smbd.mkdir(dst, service)
+                    smbd.mkdir(dst, session_info, service)
+
                 ntacl_sddl_str = _read_ntacl_file(src)
-                ntacls_helper.setntacl(dst, ntacl_sddl_str)
+                if ntacl_sddl_str:
+                    ntacls_helper.setntacl(dst, ntacl_sddl_str, session_info)
+                else:
+                    logger.warning(
+                        'Failed to restore ntacl for directory %s.' % dst
+                        + ' Please check the permissions are correct')
 
         for filename in filenames:
             if not filename.endswith('.NTACL'):
@@ -594,9 +658,14 @@ def backup_restore(src_tarfile_path, dst_service_path, samdb_conn, smb_conf_path
                 dst = os.path.join(dst_dirpath, filename)
                 if not os.path.isfile(dst):
                     # dst must be absolute path for smbd API
-                    smbd.create_file(dst, service)
+                    smbd.create_file(dst, session_info, service)
+
                 ntacl_sddl_str = _read_ntacl_file(src)
-                ntacls_helper.setntacl(dst, ntacl_sddl_str)
+                if ntacl_sddl_str:
+                    ntacls_helper.setntacl(dst, ntacl_sddl_str, session_info)
+                else:
+                    logger.warning('Failed to restore ntacl for file %s.' % dst
+                                 + ' Please check the permissions are correct')
 
                 # now put data in
                 with open(src, 'rb') as src_file:
