@@ -29,6 +29,7 @@
 #include "util_tdb.h"
 #include "../libcli/security/security.h"
 #include "groupdb/mapping_tdb.h"
+#include "lib/util/smb_strtox.h"
 
 static struct db_context *db; /* used for driver files */
 
@@ -457,7 +458,7 @@ static NTSTATUS one_alias_membership(const struct dom_sid *member,
 	char *string_sid;
 	TDB_DATA dbuf;
 	const char *p;
-	NTSTATUS status = NT_STATUS_OK;
+	NTSTATUS status;
 	TALLOC_CTX *frame = talloc_stackframe();
 
 	slprintf(key, sizeof(key), "%s%s", MEMBEROF_PREFIX,
