@@ -46,12 +46,10 @@ bool leases_db_init(bool read_only)
 		return false;
 	}
 
-	leases_db = db_open(NULL, db_path, 0,
-			    TDB_DEFAULT|
-			    TDB_VOLATILE|
-			    TDB_CLEAR_IF_FIRST|
-			    TDB_SEQNUM|
-			    TDB_INCOMPATIBLE_HASH,
+	leases_db = db_open(NULL, db_path,
+			    SMBD_VOLATILE_TDB_HASH_SIZE,
+			    SMBD_VOLATILE_TDB_FLAGS |
+			    TDB_SEQNUM,
 			    read_only ? O_RDONLY : O_RDWR|O_CREAT, 0644,
 			    DBWRAP_LOCK_ORDER_4, DBWRAP_FLAG_NONE);
 	TALLOC_FREE(db_path);
@@ -489,6 +487,11 @@ static void leases_db_set_fn(
 	value->lease_version = state->lease_version;
 	value->epoch = state->epoch;
 	*modified = true;
+
+	if (CHECK_DEBUGLVL(10)) {
+		DBG_DEBUG("\n");
+		NDR_PRINT_DEBUG(leases_db_value, value);
+	}
 }
 
 NTSTATUS leases_db_set(const struct GUID *client_guid,
