@@ -213,7 +213,7 @@ static void exit_server_common(enum server_exit_reason how,
 	client = NULL;
 	netlogon_creds_cli_close_global_db();
 	TALLOC_FREE(global_smbXsrv_client);
-	smbprofile_dump();
+	smbprofile_dump(NULL);
 	global_messaging_context_free();
 	global_event_context_free();
 	TALLOC_FREE(smbd_memcache_ctx);
