@@ -96,7 +96,7 @@ enum protocol_types smbd_smb2_protocol_dialect_match(const uint8_t *indyn,
 				const int dialect_count,
 				uint16_t *dialect)
 {
-	struct {
+	static const struct {
 		enum protocol_types proto;
 		uint16_t dialect;
 	} pd[] = {
@@ -129,6 +129,61 @@ enum protocol_types smbd_smb2_protocol_dialect_match(const uint8_t *indyn,
 	return PROTOCOL_NONE;
 }
 
+static NTSTATUS smb2_negotiate_context_process_posix(
+	const struct smb2_negotiate_contexts *in_c,
+	bool *posix)
+{
+	struct smb2_negotiate_context *in_posix = NULL;
+	const uint8_t *inbuf = NULL;
+	size_t inbuflen;
+	bool posix_found = false;
+	size_t ofs;
+	int cmp;
+
+	*posix = false;
+
+	if (!lp_smb3_unix_extensions(GLOBAL_SECTION_SNUM)) {
+		return NT_STATUS_OK;
+	}
+
+	in_posix = smb2_negotiate_context_find(in_c,
+					       SMB2_POSIX_EXTENSIONS_AVAILABLE);
+	if (in_posix == NULL) {
+		return NT_STATUS_OK;
+	}
+
+	inbuf = in_posix->data.data;
+	inbuflen = in_posix->data.length;
+
+	/*
+	 * For now the server only supports one variant.
+	 * Check it's the right one.
+	 */
+	if ((inbuflen % 16) != 0) {
+		return NT_STATUS_INVALID_PARAMETER;
+	}
+
+	SMB_ASSERT(strlen(SMB2_CREATE_TAG_POSIX) == 16);
+
+	for (ofs = 0; ofs < inbuflen; ofs += 16) {
+		cmp = memcmp(inbuf+ofs, SMB2_CREATE_TAG_POSIX, 16);
+		if (cmp == 0) {
+			posix_found = true;
+			break;
+		}
+	}
+
+	if (!posix_found) {
+		DBG_DEBUG("Client requested unknown SMB3 Unix extensions:\n");
+		dump_data(10, inbuf, inbuflen);
+		return NT_STATUS_OK;
+	}
+
+	DBG_DEBUG("Client requested SMB3 Unix extensions\n");
+	*posix = true;
+	return NT_STATUS_OK;
+}
+
 struct smbd_smb2_request_process_negprot_state {
 	struct smbd_smb2_request *req;
 	DATA_BLOB outbody;
@@ -163,7 +218,6 @@ NTSTATUS smbd_smb2_request_process_negprot(struct smbd_smb2_request *req)
 	struct smb2_negotiate_context *in_cipher = NULL;
 	struct smb2_negotiate_context *in_sign_algo = NULL;
 	struct smb2_negotiate_contexts out_c = { .num_contexts = 0, };
-	struct smb2_negotiate_context *in_posix = NULL;
 	const struct smb311_capabilities default_smb3_capabilities =
 		smb311_capabilities_parse("server",
 			lp_server_smb3_signing_algorithms(),
@@ -181,6 +235,7 @@ NTSTATUS smbd_smb2_request_process_negprot(struct smbd_smb2_request *req)
 	uint32_t max_read = lp_smb2_max_read();
 	uint32_t max_write = lp_smb2_max_write();
 	NTTIME now = timeval_to_nttime(&req->request_time);
+	bool posix = false;
 	bool ok;
 
 	status = smbd_smb2_request_verify_sizes(req, 0x24);
@@ -276,41 +331,9 @@ NTSTATUS smbd_smb2_request_process_negprot(struct smbd_smb2_request *req)
 			return smbd_smb2_request_error(req, status);
 		}
 
-		if (lp_smb3_unix_extensions()) {
-			in_posix = smb2_negotiate_context_find(&in_c,
-					SMB2_POSIX_EXTENSIONS_AVAILABLE);
-
-			if (in_posix != NULL) {
-				const uint8_t *inbuf = in_posix->data.data;
-				size_t inbuflen = in_posix->data.length;
-				bool posix_found = false;
-				/*
-				 * For now the server only supports one variant.
-				 * Check it's the right one.
-				 */
-				if ((inbuflen % 16) != 0) {
-					return smbd_smb2_request_error(req,
-						NT_STATUS_INVALID_PARAMETER);
-				}
-				SMB_ASSERT(strlen(SMB2_CREATE_TAG_POSIX) == 16);
-				for (ofs=0; ofs<inbuflen; ofs+=16) {
-					if (memcmp(inbuf+ofs,
-							SMB2_CREATE_TAG_POSIX,
-							16) == 0) {
-						posix_found = true;
-						break;
-					}
-				}
-				if (posix_found) {
-					DBG_DEBUG("Client requested SMB2 unix "
-						"extensions\n");
-				} else {
-					DBG_DEBUG("Client requested unknown "
-						"SMB2 unix extensions:\n");
-					dump_data(10, inbuf, inbuflen);
-					in_posix = NULL;
-				}
-			}
+		status = smb2_negotiate_context_process_posix(&in_c, &posix);
+		if (!NT_STATUS_IS_OK(status)) {
+			return smbd_smb2_request_error(req, status);
 		}
 	}
 
@@ -357,7 +380,7 @@ NTSTATUS smbd_smb2_request_process_negprot(struct smbd_smb2_request *req)
 	in_sign_algo = smb2_negotiate_context_find(&in_c,
 					SMB2_SIGNING_CAPABILITIES);
 
-	/* negprot_spnego() returns a the server guid in the first 16 bytes */
+	/* negprot_spnego() returns the server guid in the first 16 bytes */
 	negprot_spnego_blob = negprot_spnego(req, xconn);
 	if (negprot_spnego_blob.data == NULL) {
 		return smbd_smb2_request_error(req, NT_STATUS_NO_MEMORY);
@@ -391,6 +414,13 @@ NTSTATUS smbd_smb2_request_process_negprot(struct smbd_smb2_request *req)
 		capabilities |= SMB2_CAP_ENCRYPTION;
 	}
 
+	if (protocol >= PROTOCOL_SMB3_00 &&
+	    in_capabilities & SMB2_CAP_DIRECTORY_LEASING &&
+	    lp_smb3_directory_leases())
+	{
+		capabilities |= SMB2_CAP_DIRECTORY_LEASING;
+	}
+
 	/*
 	 * 0x10000 (65536) is the maximum allowed message size
 	 * for SMB 2.0
@@ -688,7 +718,7 @@ NTSTATUS smbd_smb2_request_process_negprot(struct smbd_smb2_request *req)
 	security_buffer = data_blob_const(NULL, 0);
 #endif
 
-	if (in_posix != NULL) {
+	if (posix) {
 		/* Client correctly negotiated SMB2 unix extensions. */
 		const uint8_t *buf = (const uint8_t *)SMB2_CREATE_TAG_POSIX;
 		status = smb2_negotiate_context_add(
@@ -782,8 +812,6 @@ NTSTATUS smbd_smb2_request_process_negprot(struct smbd_smb2_request *req)
 	SIVAL(outbody.data, 0x3C,
 	      out_negotiate_context_offset);	/* reserved/NegotiateContextOffset */
 
-	req->sconn->using_smb2 = true;
-
 	if (dialect == SMB2_DIALECT_REVISION_2FF) {
 		return smbd_smb2_request_done(req, outbody, &outdyn);
 	}
@@ -890,10 +918,7 @@ static void smbd_smb2_request_process_negprot_mc_done(struct tevent_req *subreq)
 						NT_STATUS_CONNECTION_IN_USE);
 		smbd_server_connection_terminate(xconn,
 						 "passed connection");
-		/*
-		 * smbd_server_connection_terminate() should not return!
-		 */
-		smb_panic(__location__);
+		exit_server_cleanly("connection passed");
 		return;
 	}
 	if (!NT_STATUS_IS_OK(status)) {
@@ -906,10 +931,7 @@ static void smbd_smb2_request_process_negprot_mc_done(struct tevent_req *subreq)
 		 * The connection was passed to another process
 		 */
 		smbd_server_connection_terminate(xconn, nt_errstr(status));
-		/*
-		 * smbd_server_connection_terminate() should not return!
-		 */
-		smb_panic(__location__);
+		exit_server_cleanly("connection passed");
 		return;
 	}
 
@@ -938,10 +960,7 @@ static void smbd_smb2_request_process_negprot_mc_done(struct tevent_req *subreq)
 	 * The connection was passed to another process
 	 */
 	smbd_server_connection_terminate(xconn, nt_errstr(status));
-	/*
-	 * smbd_server_connection_terminate() should not return!
-	 */
-	smb_panic(__location__);
+	exit_server_cleanly("connection passed");
 	return;
 }
 
