@@ -240,6 +240,7 @@ static bool b9_format(struct dlz_bind9_data *state,
 		break;
 
 	case DNS_TYPE_SOA: {
+		const char *dns_hostname = NULL;
 		const char *mname;
 		*type = "soa";
 
@@ -247,13 +248,11 @@ static bool b9_format(struct dlz_bind9_data *state,
 		 * point at ourselves. This is how AD DNS servers
 		 * force clients to send updates to the right local DC
 		 */
-		mname = talloc_asprintf(mem_ctx, "%s.%s.",
-					lpcfg_netbios_name(state->lp),
-					lpcfg_dnsdomain(state->lp));
-		if (mname == NULL) {
+		dns_hostname = lpcfg_dns_hostname(state->lp);
+		if (dns_hostname == NULL) {
 			return false;
 		}
-		mname = strlower_talloc(mem_ctx, mname);
+		mname = talloc_asprintf(mem_ctx, "%s.", dns_hostname);
 		if (mname == NULL) {
 			return false;
 		}
@@ -672,6 +671,9 @@ _PUBLIC_ isc_result_t dlz_create(const char *dlzname,
 	}
 	va_end(ap);
 
+	/* starting from here, we can only use state->log() until
+	 * dlz_bind9_state is assigned */
+
 	/* Do not install samba signal handlers */
 	fault_setup_disable();
 
@@ -728,6 +730,11 @@ _PUBLIC_ isc_result_t dlz_create(const char *dlzname,
 		}
 
 		if (!file_exist(state->options.url)) {
+			state->log(ISC_LOG_ERROR,
+				   "samba_dlz: dlz_create could not find '%s'; "
+				   "trying old location '%s/dns/sam.ldb' instead",
+				   state->options.url,
+				   lpcfg_private_dir(state->lp));
 			state->options.url = talloc_asprintf(state,
 							     "%s/dns/sam.ldb",
 							     lpcfg_private_dir(state->lp));
