@@ -293,12 +293,8 @@ static void prefork_fork_master(
 	}
 
 	pid = getpid();
-	setproctitle("task[%s] pre-fork master", service_name);
-	/*
-	 * We must fit within 15 chars of text or we will truncate, so
-	 * we put the constant part last
-	 */
-	prctl_set_comment("%s[master]", service_name);
+
+	process_set_title("%s[master]", "task[%s] pre-fork master", service_name);
 
 	/*
 	 * this will free all the listening sockets and all state that
@@ -319,6 +315,9 @@ static void prefork_fork_master(
 		if (task != NULL && service_details->post_fork != NULL) {
 			service_details->post_fork(task, &pd);
 		}
+		if (task != NULL && service_details->before_loop != NULL) {
+			service_details->before_loop(task);
+		}
 		tevent_loop_wait(ev);
 		TALLOC_FREE(ev);
 		exit(0);
@@ -364,7 +363,7 @@ static void prefork_fork_master(
 		struct talloc_ctx *ctx = talloc_new(NULL);
 		char *name = NULL;
 		if (ctx == NULL) {
-			DBG_ERR("Out of memory");
+			DBG_ERR("Out of memory\n");
 			exit(127);
 		}
 		name = talloc_asprintf(ctx, "prefork-master-%s", service_name);
@@ -402,6 +401,51 @@ static void prefork_fork_master(
 		smb_set_close_on_exec(control_pipe[1]);
 	}
 
+	/*
+	 * Note, we call this before the first
+	 * prefork_fork_worker() in order to have
+	 * a stable order of:
+	 *  task_init(master) -> before_loop(master)
+	 *  -> post_fork(worker) -> before_loop(worker)
+	 *
+	 * Otherwise we would have different behaviors
+	 * between the first prefork_fork_worker() loop
+	 * and restarting of died workers
+	 */
+	if (task != NULL && service_details->before_loop != NULL) {
+		struct task_server *task_copy = NULL;
+
+		/*
+		 * We need to use ev as parent in order to
+		 * keep everything alive during the loop
+		 */
+		task_copy = talloc(ev, struct task_server);
+		if (task_copy == NULL) {
+			TALLOC_FREE(ev);
+			TALLOC_FREE(ev2);
+			exit(127);
+		}
+		*task_copy = *task;
+
+		/*
+		 * In order to allow the before_loop() hook
+		 * to register messages or event handlers,
+		 * we need to fix up task->event_ctx
+		 * and create a new task->msg_ctx
+		 */
+		task_copy->event_ctx = ev;
+		task_copy->msg_ctx = imessaging_init(task_copy,
+						task_copy->lp_ctx,
+						task_copy->server_id,
+						task_copy->event_ctx);
+		if (task_copy->msg_ctx == NULL) {
+			TALLOC_FREE(ev);
+			TALLOC_FREE(ev2);
+			exit(127);
+		}
+		service_details->before_loop(task_copy);
+	}
+
 	/*
 	 * We are now free to spawn some worker processes
 	 */
@@ -418,8 +462,19 @@ static void prefork_fork_master(
 		pd.instances++;
 	}
 
+	/*
+	 * Make sure the messaging context
+	 * used by the workers is no longer
+	 * active on ev2, otherwise we
+	 * would have memory leaks, because
+	 * we queue incoming messages
+	 * and never process them via ev2.
+	 */
+	imessaging_dgm_unref_ev(ev2);
+
 	/* Don't listen on the sockets we just gave to the children */
 	tevent_loop_wait(ev);
+	imessaging_dgm_unref_ev(ev);
 	TALLOC_FREE(ev);
 	/* We need to keep ev2 until we're finished for the messaging to work */
 	TALLOC_FREE(ev2);
@@ -566,21 +621,24 @@ static void prefork_child_pipe_handler(struct tevent_context *ev,
 	irpc_cleanup(rc->lp_ctx, ev, pid);
 	status = tfork_status(&rc->t, false);
 	if (status == -1) {
-		DBG_ERR("Parent %d, Child %d terminated, "
+		DBG_ERR("%s: Parent %d, Child %d terminated, "
 			"unable to get status code from tfork\n",
-			getpid(), pid);
+			rc->service_name, getpid(), pid);
 		rc_inuse = prefork_restart(ev, rc);
 	} else if (WIFEXITED(status)) {
 		status = WEXITSTATUS(status);
-		DBG_ERR("Parent %d, Child %d exited with status %d\n",
-			 getpid(), pid,  status);
-		if (status != 0) {
+		if (status == 0) {
+			DBG_WARNING("%s: Parent %d, Child %d exited with status 0\n",
+				    rc->service_name, getpid(), pid);
+		} else {
+			DBG_ERR("%s: Parent %d, Child %d exited with status %d\n",
+				rc->service_name, getpid(), pid,  status);
 			rc_inuse = prefork_restart(ev, rc);
 		}
 	} else if (WIFSIGNALED(status)) {
 		status = WTERMSIG(status);
-		DBG_ERR("Parent %d, Child %d terminated with signal %d\n",
-			getpid(), pid, status);
+		DBG_ERR("%s: Parent %d, Child %d terminated with signal %d\n",
+			rc->service_name, getpid(), pid, status);
 		if (status == SIGABRT || status == SIGBUS || status == SIGFPE ||
 		    status == SIGILL || status == SIGSYS || status == SIGSEGV ||
 		    status == SIGKILL) {
@@ -752,17 +810,14 @@ static void prefork_fork_worker(struct task_server *task,
 		 */
 		free(w);
 
+		imessaging_dgm_unref_ev(ev);
 		TALLOC_FREE(ev);
-		setproctitle("task[%s] pre-forked worker(%d)",
-			     service_name,
-			     pd->instances);
-		/*
-		 * We must fit within 15 chars of text or we will truncate, so
-		 * we put child number last
-		 */
-		prctl_set_comment("%s(%d)",
+
+		process_set_title("%s(%d)",
+				  "task[%s] pre-forked worker(%d)",
 				  service_name,
 				  pd->instances);
+
 		prefork_reload_after_fork();
 		if (service_details->post_fork != NULL) {
 			service_details->post_fork(task, pd);
@@ -780,7 +835,11 @@ static void prefork_fork_worker(struct task_server *task,
 			irpc_add_name(task->msg_ctx, name);
 			TALLOC_FREE(ctx);
 		}
+		if (service_details->before_loop != NULL) {
+			service_details->before_loop(task);
+		}
 		tevent_loop_wait(ev2);
+		imessaging_dgm_unref_ev(ev2);
 		talloc_free(ev2);
 		exit(0);
 	}
