@@ -22,7 +22,7 @@
 #include "includes.h"
 #include "torture/rpc/torture_rpc.h"
 #include "libnet/libnet.h"
-#include "lib/cmdline/popt_common.h"
+#include "lib/cmdline/cmdline.h"
 #include "librpc/gen_ndr/ndr_srvsvc_c.h"
 #include "torture/libnet/proto.h"
 
@@ -175,14 +175,14 @@ bool torture_listshares(struct torture_context *torture)
 		goto done;
 	}
 
-	libnetctx = libnet_context_init(torture->ev, torture->lp_ctx);
+	libnetctx = libnet_context_init(torture, torture->ev, torture->lp_ctx);
 	if (!libnetctx) {
 		torture_comment(torture, "Couldn't allocate libnet context\n");
 		ret = false;
 		goto done;
 	}
 
-	libnetctx->cred = popt_get_cmdline_credentials();
+	libnetctx->cred = samba_cmdline_get_creds();
 
 	torture_comment(torture, "Testing libnet_ListShare\n");
 
@@ -262,8 +262,8 @@ bool torture_delshare(struct torture_context *torture)
 	status = torture_rpc_binding(torture, &binding);
 	torture_assert_ntstatus_ok(torture, status, "Failed to get binding");
 
-	libnetctx = libnet_context_init(torture->ev, torture->lp_ctx);
-	libnetctx->cred = popt_get_cmdline_credentials();
+	libnetctx = libnet_context_init(torture, torture->ev, torture->lp_ctx);
+	libnetctx->cred = samba_cmdline_get_creds();
 
 	status = torture_rpc_connection(torture,
 					&p,
