@@ -20,7 +20,7 @@
 #include "source3/include/includes.h"
 #include "torture/smbtorture.h"
 #include "auth/credentials/credentials.h"
-#include "lib/cmdline/popt_common.h"
+#include "lib/cmdline/cmdline.h"
 #include "source3/lib/netapi/netapi.h"
 #include "source3/lib/netapi/netapi_private.h"
 #include "lib/param/param.h"
@@ -32,26 +32,22 @@ bool torture_libnetapi_init_context(struct torture_context *tctx,
 	NET_API_STATUS status;
 	struct libnetapi_ctx *ctx;
 	TALLOC_CTX *frame = talloc_stackframe();
+	struct cli_credentials *creds = samba_cmdline_get_creds();
 
-	if (!lp_load(lpcfg_configfile(tctx->lp_ctx), true, false, false, true)) {
+	if (!lp_load_global(lpcfg_configfile(tctx->lp_ctx))) {
 		fprintf(stderr, "error loading %s\n", lpcfg_configfile(tctx->lp_ctx));
-		return W_ERROR_V(WERR_GENERAL_FAILURE);
+		talloc_free(frame);
+		return W_ERROR_V(WERR_GEN_FAILURE);
 	}
 
-	init_names();
 	load_interfaces();
 
-	status = libnetapi_net_init(&ctx);
+	status = libnetapi_net_init(&ctx, tctx->lp_ctx, creds);
 	if (status != 0) {
 		talloc_free(frame);
 		return false;
 	}
 
-	libnetapi_set_username(ctx,
-		cli_credentials_get_username(cmdline_credentials));
-	libnetapi_set_password(ctx,
-		cli_credentials_get_password(cmdline_credentials));
-
 	*ctx_p = ctx;
 
 	talloc_free(frame);
@@ -72,18 +68,18 @@ static bool torture_libnetapi_initialize(struct torture_context *tctx)
 	status = libnetapi_init(&ctx);
 
 	torture_assert(tctx, ctx != NULL, "Failed to get a libnetapi_ctx");
-	torture_assert_int_equal(tctx, status, 0, "libnetapi_init failed despite alredy being set up");
+	torture_assert_int_equal(tctx, status, 0, "libnetapi_init failed despite already being set up");
 
 	libnetapi_free(ctx);
 
 	return true;
 }
 
-NTSTATUS torture_libnetapi_init(void)
+NTSTATUS torture_libnetapi_init(TALLOC_CTX *ctx)
 {
 	struct torture_suite *suite;
 
-	suite = torture_suite_create(talloc_autofree_context(), "netapi");
+	suite = torture_suite_create(ctx, "netapi");
 
 	torture_suite_add_simple_test(suite, "server", torture_libnetapi_server);
 	torture_suite_add_simple_test(suite, "group", torture_libnetapi_group);
@@ -92,7 +88,7 @@ NTSTATUS torture_libnetapi_init(void)
 
 	suite->description = talloc_strdup(suite, "libnetapi convenience interface tests");
 
-	torture_register_suite(suite);
+	torture_register_suite(ctx, suite);
 
 	return NT_STATUS_OK;
 }
