@@ -22,6 +22,8 @@
 #include <netapi.h>
 #include "torture/libnetapi/proto.h"
 
+#undef strcasecmp
+
 #define TORTURE_TEST_USER "torture_testuser"
 #define TORTURE_TEST_USER2 "torture_testuser2"
 
@@ -271,8 +273,8 @@ static NET_API_STATUS test_netusergetgroups(struct torture_context *tctx,
 	uint8_t *buffer = NULL;
 	int i;
 
-	struct GROUP_USERS_INFO_0 *i0;
-	struct GROUP_USERS_INFO_1 *i1;
+	struct GROUP_USERS_INFO_0 *i0 = NULL;
+	struct GROUP_USERS_INFO_1 *i1 = NULL;
 
 	torture_comment(tctx, "Testing NetUserGetGroups level %d\n", level);
 
