@@ -31,6 +31,10 @@
 #include "torture/nbt/proto.h"
 #include "param/param.h"
 
+/* rcode used when you don't want to check the rcode */
+#define WINS_TEST_RCODE_WE_DONT_CARE 255
+
+
 #define CHECK_VALUE(tctx, v, correct) \
 	torture_assert_int_equal(tctx, v, correct, "Incorrect value")
 
@@ -49,7 +53,8 @@
 */
 static bool nbt_test_wins_name(struct torture_context *tctx, const char *address,
 			       struct nbt_name *name, uint16_t nb_flags,
-			       bool try_low_port)
+			       bool try_low_port,
+			       uint8_t register_rcode)
 {
 	struct nbt_name_register_wins io;
 	struct nbt_name_register name_register;
@@ -63,14 +68,15 @@ static bool nbt_test_wins_name(struct torture_context *tctx, const char *address
 	struct socket_address *socket_address;
 	struct interface *ifaces;
 	bool low_port = try_low_port;
+	char **l;
 
-	load_interfaces(tctx, lp_interfaces(tctx->lp_ctx), &ifaces);
+	load_interface_list(tctx, tctx->lp_ctx, &ifaces);
 
-	myaddress = talloc_strdup(tctx, iface_best_ip(ifaces, address));
+	myaddress = talloc_strdup(tctx, iface_list_best_ip(ifaces, address));
 
 	socket_address = socket_address_from_strings(tctx, 
 						     nbtsock->sock->backend_name,
-						     myaddress, lp_nbt_port(tctx->lp_ctx));
+						     myaddress, lpcfg_nbt_port(tctx->lp_ctx));
 	torture_assert(tctx, socket_address != NULL, 
 				   "Error getting address");
 
@@ -97,7 +103,7 @@ static bool nbt_test_wins_name(struct torture_context *tctx, const char *address
 
 	torture_comment(tctx, "release the name\n");
 	release.in.name = *name;
-	release.in.dest_port = lp_nbt_port(tctx->lp_ctx);
+	release.in.dest_port = lpcfg_nbt_port(tctx->lp_ctx);
 	release.in.dest_addr = address;
 	release.in.address = myaddress;
 	release.in.nb_flags = nb_flags;
@@ -116,9 +122,11 @@ static bool nbt_test_wins_name(struct torture_context *tctx, const char *address
 	} else {
 		torture_comment(tctx, "register the name with a wrong address (makes the next request slow!)\n");
 		io.in.name = *name;
-		io.in.wins_port = lp_nbt_port(tctx->lp_ctx);
-		io.in.wins_servers = str_list_make_single(tctx, address);
-		io.in.addresses = str_list_make_single(tctx, "127.64.64.1");
+		io.in.wins_port = lpcfg_nbt_port(tctx->lp_ctx);
+		io.in.wins_servers = const_str_list(
+			str_list_make_single(tctx, address));
+		io.in.addresses = const_str_list(
+			str_list_make_single(tctx, "127.64.64.1"));
 		io.in.nb_flags = nb_flags;
 		io.in.ttl = 300000;
 
@@ -133,11 +141,13 @@ static bool nbt_test_wins_name(struct torture_context *tctx, const char *address
 					address));
 
 		CHECK_STRING(tctx, io.out.wins_server, address);
-		CHECK_VALUE(tctx, io.out.rcode, 0);
+		if (register_rcode != WINS_TEST_RCODE_WE_DONT_CARE) {
+			CHECK_VALUE(tctx, io.out.rcode, 0);
+		}
 
 		torture_comment(tctx, "register the name correct address\n");
 		name_register.in.name		= *name;
-		name_register.in.dest_port	= lp_nbt_port(tctx->lp_ctx);
+		name_register.in.dest_port	= lpcfg_nbt_port(tctx->lp_ctx);
 		name_register.in.dest_addr	= address;
 		name_register.in.address	= myaddress;
 		name_register.in.nb_flags	= nb_flags;
@@ -153,7 +163,7 @@ static bool nbt_test_wins_name(struct torture_context *tctx, const char *address
 		 */
 		req = nbt_name_register_send(nbtsock, &name_register);
 		while (true) {
-			event_loop_once(nbtsock->event_ctx);
+			tevent_loop_once(nbtsock->event_ctx);
 			if (req->state != NBT_REQUEST_WAIT) {
 				break;
 			}
@@ -165,9 +175,8 @@ static bool nbt_test_wins_name(struct torture_context *tctx, const char *address
 				 * and not handle it as new request
 				 */
 				req->state = NBT_REQUEST_SEND;
-				DLIST_ADD_END(nbtsock->send_queue, req,
-					      struct nbt_name_request *);
-				EVENT_FD_WRITEABLE(nbtsock->fde);
+				DLIST_ADD_END(nbtsock->send_queue, req);
+				TEVENT_FD_WRITEABLE(nbtsock->fde);
 				break;
 			}
 		}
@@ -182,15 +191,19 @@ static bool nbt_test_wins_name(struct torture_context *tctx, const char *address
 			talloc_asprintf(tctx, "Bad response from %s for name register\n",
 					address));
 
-		CHECK_VALUE(tctx, name_register.out.rcode, 0);
+		if (register_rcode != WINS_TEST_RCODE_WE_DONT_CARE) {
+			CHECK_VALUE(tctx, name_register.out.rcode, 0);
+		}
 		CHECK_STRING(tctx, name_register.out.reply_addr, myaddress);
 	}
 
 	torture_comment(tctx, "register the name correct address\n");
 	io.in.name = *name;
-	io.in.wins_port = lp_nbt_port(tctx->lp_ctx);
-	io.in.wins_servers = (const char **)str_list_make_single(tctx, address);
-	io.in.addresses = (const char **)str_list_make_single(tctx, myaddress);
+	io.in.wins_port = lpcfg_nbt_port(tctx->lp_ctx);
+	l = str_list_make_single(tctx, address);
+	io.in.wins_servers = discard_const_p(const char *, l);
+	l = str_list_make_single(tctx, myaddress);
+	io.in.addresses = discard_const_p(const char *, l);
 	io.in.nb_flags = nb_flags;
 	io.in.ttl = 300000;
 	
@@ -198,7 +211,13 @@ static bool nbt_test_wins_name(struct torture_context *tctx, const char *address
 	torture_assert_ntstatus_ok(tctx, status, talloc_asprintf(tctx, "Bad response from %s for name register", address));
 	
 	CHECK_STRING(tctx, io.out.wins_server, address);
-	CHECK_VALUE(tctx, io.out.rcode, 0);
+	if (register_rcode != WINS_TEST_RCODE_WE_DONT_CARE) {
+		CHECK_VALUE(tctx, io.out.rcode, register_rcode);
+	}
+
+	if (register_rcode != NBT_RCODE_OK) {
+		return true;
+	}
 
 	if (name->type != NBT_NAME_MASTER &&
 	    name->type != NBT_NAME_LOGON && 
@@ -215,7 +234,7 @@ static bool nbt_test_wins_name(struct torture_context *tctx, const char *address
 	torture_comment(tctx, "query the name to make sure its there\n");
 	query.in.name = *name;
 	query.in.dest_addr = address;
-	query.in.dest_port = lp_nbt_port(tctx->lp_ctx);
+	query.in.dest_port = lpcfg_nbt_port(tctx->lp_ctx);
 	query.in.broadcast = false;
 	query.in.wins_lookup = true;
 	query.in.timeout = 3;
@@ -261,9 +280,11 @@ static bool nbt_test_wins_name(struct torture_context *tctx, const char *address
 
 	torture_comment(tctx, "refresh the name\n");
 	refresh.in.name = *name;
-	refresh.in.wins_port = lp_nbt_port(tctx->lp_ctx);
-	refresh.in.wins_servers = (const char **)str_list_make_single(tctx, address);
-	refresh.in.addresses = (const char **)str_list_make_single(tctx, myaddress);
+	refresh.in.wins_port = lpcfg_nbt_port(tctx->lp_ctx);
+	l = str_list_make_single(tctx, address);
+	refresh.in.wins_servers = discard_const_p(const char *, l);
+	l = str_list_make_single(tctx, myaddress);
+	refresh.in.addresses = discard_const_p(const char *, l);
 	refresh.in.nb_flags = nb_flags;
 	refresh.in.ttl = 12345;
 	
@@ -282,7 +303,7 @@ static bool nbt_test_wins_name(struct torture_context *tctx, const char *address
 
 	printf("release the name\n");
 	release.in.name = *name;
-	release.in.dest_port = lp_nbt_port(tctx->lp_ctx);
+	release.in.dest_port = lpcfg_nbt_port(tctx->lp_ctx);
 	release.in.dest_addr = address;
 	release.in.address = myaddress;
 	release.in.nb_flags = nb_flags;
@@ -310,9 +331,11 @@ static bool nbt_test_wins_name(struct torture_context *tctx, const char *address
 	} else {
 		torture_comment(tctx, "register the name with a wrong address (makes the next request slow!)\n");
 		io.in.name = *name;
-		io.in.wins_port = lp_nbt_port(tctx->lp_ctx);
-		io.in.wins_servers = str_list_make_single(tctx, address);
-		io.in.addresses = str_list_make_single(tctx, "127.64.64.1");
+		io.in.wins_port = lpcfg_nbt_port(tctx->lp_ctx);
+		io.in.wins_servers = const_str_list(
+			str_list_make_single(tctx, address));
+		io.in.addresses = const_str_list(
+			str_list_make_single(tctx, "127.64.64.1"));
 		io.in.nb_flags = nb_flags;
 		io.in.ttl = 300000;
 	
@@ -332,9 +355,11 @@ static bool nbt_test_wins_name(struct torture_context *tctx, const char *address
 
 	torture_comment(tctx, "refresh the name with the correct address\n");
 	refresh.in.name = *name;
-	refresh.in.wins_port = lp_nbt_port(tctx->lp_ctx);
-	refresh.in.wins_servers = str_list_make_single(tctx, address);
-	refresh.in.addresses = str_list_make_single(tctx, myaddress);
+	refresh.in.wins_port = lpcfg_nbt_port(tctx->lp_ctx);
+	refresh.in.wins_servers = const_str_list(
+			str_list_make_single(tctx, address));
+	refresh.in.addresses = const_str_list(
+			str_list_make_single(tctx, myaddress));
 	refresh.in.nb_flags = nb_flags;
 	refresh.in.ttl = 12345;
 
@@ -353,7 +378,7 @@ static bool nbt_test_wins_name(struct torture_context *tctx, const char *address
 
 	torture_comment(tctx, "release the name\n");
 	release.in.name = *name;
-	release.in.dest_port = lp_nbt_port(tctx->lp_ctx);
+	release.in.dest_port = lpcfg_nbt_port(tctx->lp_ctx);
 	release.in.dest_addr = address;
 	release.in.address = myaddress;
 	release.in.nb_flags = nb_flags;
@@ -394,6 +419,35 @@ static bool nbt_test_wins_name(struct torture_context *tctx, const char *address
 }
 
 
+static char *test_nbt_wins_scope_string(TALLOC_CTX *mem_ctx, uint8_t count)
+{
+	char *res;
+	uint8_t i;
+
+	res = talloc_array(mem_ctx, char, count+1);
+	if (res == NULL) {
+		return NULL;
+	}
+
+	for (i=0; i < count; i++) {
+		switch (i) {
+		case 63:
+		case 63 + 1 + 63:
+		case 63 + 1 + 63 + 1 + 63:
+			res[i] = '.';
+			break;
+		default:
+			res[i] = '0' + (i%10);
+			break;
+		}
+	}
+
+	res[count] = '\0';
+
+	talloc_set_name_const(res, res);
+
+	return res;
+}
 
 /*
   test operations against a WINS server
@@ -412,66 +466,210 @@ static bool nbt_test_wins(struct torture_context *tctx)
 
 	name.type = NBT_NAME_CLIENT;
 	name.scope = NULL;
-	ret &= nbt_test_wins_name(tctx, address, &name, NBT_NODE_H, true);
+	ret &= nbt_test_wins_name(tctx, address, &name,
+				  NBT_NODE_H, true, NBT_RCODE_OK);
 
 	name.type = NBT_NAME_MASTER;
-	ret &= nbt_test_wins_name(tctx, address, &name, NBT_NODE_H, false);
+	ret &= nbt_test_wins_name(tctx, address, &name,
+				  NBT_NODE_H, false, NBT_RCODE_OK);
 
-	ret &= nbt_test_wins_name(tctx, address, &name, NBT_NODE_H | NBT_NM_GROUP, false);
+	ret &= nbt_test_wins_name(tctx, address, &name,
+				  NBT_NODE_H | NBT_NM_GROUP, false, NBT_RCODE_OK);
 
 	name.type = NBT_NAME_SERVER;
-	ret &= nbt_test_wins_name(tctx, address, &name, NBT_NODE_H, true);
+	ret &= nbt_test_wins_name(tctx, address, &name,
+				  NBT_NODE_H, true, NBT_RCODE_OK);
 
 	name.type = NBT_NAME_LOGON;
-	ret &= nbt_test_wins_name(tctx, address, &name, NBT_NODE_H | NBT_NM_GROUP, false);
+	ret &= nbt_test_wins_name(tctx, address, &name,
+				  NBT_NODE_H | NBT_NM_GROUP, false, NBT_RCODE_OK);
 
 	name.type = NBT_NAME_BROWSER;
-	ret &= nbt_test_wins_name(tctx, address, &name, NBT_NODE_H | NBT_NM_GROUP, false);
+	ret &= nbt_test_wins_name(tctx, address, &name,
+				  NBT_NODE_H | NBT_NM_GROUP, false, NBT_RCODE_OK);
 
 	name.type = NBT_NAME_PDC;
-	ret &= nbt_test_wins_name(tctx, address, &name, NBT_NODE_H, true);
+	ret &= nbt_test_wins_name(tctx, address, &name,
+				  NBT_NODE_H, true, NBT_RCODE_OK);
 
 	name.type = 0xBF;
-	ret &= nbt_test_wins_name(tctx, address, &name, NBT_NODE_H, true);
+	ret &= nbt_test_wins_name(tctx, address, &name,
+				  NBT_NODE_H, true, NBT_RCODE_OK);
 
 	name.type = 0xBE;
-	ret &= nbt_test_wins_name(tctx, address, &name, NBT_NODE_H, false);
+	ret &= nbt_test_wins_name(tctx, address, &name,
+				  NBT_NODE_H, false, NBT_RCODE_OK);
 
 	name.scope = "example";
 	name.type = 0x72;
-	ret &= nbt_test_wins_name(tctx, address, &name, NBT_NODE_H, true);
+	ret &= nbt_test_wins_name(tctx, address, &name,
+				  NBT_NODE_H, true, NBT_RCODE_OK);
 
 	name.scope = "example";
 	name.type = 0x71;
-	ret &= nbt_test_wins_name(tctx, address, &name, NBT_NODE_H | NBT_NM_GROUP, false);
+	ret &= nbt_test_wins_name(tctx, address, &name,
+				  NBT_NODE_H | NBT_NM_GROUP, false, NBT_RCODE_OK);
 
 	name.scope = "foo.example.com";
 	name.type = 0x72;
-	ret &= nbt_test_wins_name(tctx, address, &name, NBT_NODE_H, false);
+	ret &= nbt_test_wins_name(tctx, address, &name,
+				  NBT_NODE_H, false, NBT_RCODE_OK);
 
 	name.name = talloc_asprintf(tctx, "_T\01-%5u.foo", r);
-	ret &= nbt_test_wins_name(tctx, address, &name, NBT_NODE_H, false);
+	ret &= nbt_test_wins_name(tctx, address, &name,
+				  NBT_NODE_H, false, NBT_RCODE_OK);
 
 	name.name = "";
-	ret &= nbt_test_wins_name(tctx, address, &name, NBT_NODE_H, false);
+	ret &= nbt_test_wins_name(tctx, address, &name,
+				  NBT_NODE_H, false, NBT_RCODE_OK);
 
 	name.name = talloc_asprintf(tctx, ".");
-	ret &= nbt_test_wins_name(tctx, address, &name, NBT_NODE_H, false);
+	ret &= nbt_test_wins_name(tctx, address, &name,
+				  NBT_NODE_H, false, NBT_RCODE_OK);
 
 	name.name = talloc_asprintf(tctx, "%5u-\377\200\300FOO", r);
-	ret &= nbt_test_wins_name(tctx, address, &name, NBT_NODE_H, false);
+	ret &= nbt_test_wins_name(tctx, address, &name,
+				  NBT_NODE_H, false, NBT_RCODE_OK);
+
+	name.scope = test_nbt_wins_scope_string(tctx, 237);
+	ret &= nbt_test_wins_name(tctx, address, &name,
+				  NBT_NODE_H, false, NBT_RCODE_OK);
+
+	name.scope = test_nbt_wins_scope_string(tctx, 238);
+	ret &= nbt_test_wins_name(tctx, address, &name,
+				  NBT_NODE_H, false, NBT_RCODE_SVR);
 
 	return ret;
 }
 
+/*
+ * Test that the WINS server does not call 'wins hook' when the name
+ * contains dodgy characters.
+ */
+static bool nbt_test_wins_bad_names(struct torture_context *tctx)
+{
+	const char *address = NULL;
+	const char *wins_hook_file = NULL;
+	bool ret = true;
+	int err;
+	bool ok;
+	struct nbt_name name = {};
+	size_t i, j;
+	FILE *fh = NULL;
+
+	struct {
+		const char *name;
+		bool should_succeed;
+	} test_cases[] = {
+		{"NORMAL", true},
+		{"|look|", false},
+		{"look&true", false},
+		{"look\\;false", false},
+		{"&ls>foo", false},  /* already fails due to DN syntax */
+		{"has spaces", false},
+		{"hyphen-dot.0", true},
+	};
+
+	wins_hook_file = talloc_asprintf(tctx, "%s/wins_hook_writes_here",
+					 getenv("SELFTEST_TMPDIR"));
+
+	if (!torture_nbt_get_name(tctx, &name, &address)) {
+		return false;
+	}
+
+	for (i = 0; i < ARRAY_SIZE(test_cases); i++) {
+		err =  unlink(wins_hook_file);
+		if (err != 0 && errno != ENOENT) {
+			/* we expect ENOENT, but nothing else */
+			torture_comment(tctx,
+					"unlink %zu of '%s' failed\n",
+					i, wins_hook_file);
+		}
+
+		name.name = test_cases[i].name;
+		name.type = NBT_NAME_CLIENT;
+		ok = nbt_test_wins_name(tctx, address,
+					&name,
+					NBT_NODE_H,
+					true,
+					WINS_TEST_RCODE_WE_DONT_CARE
+			);
+		if (ok == false) {
+			/*
+			 * This happens when the name interferes with
+			 * the DN syntax when it is put in winsdb.
+			 *
+			 * The wins hook will not be reached.
+			 */
+			torture_comment(tctx, "tests for '%s' failed\n",
+					name.name);
+		}
+
+		/*
+		 * poll for the file being created by the wins hook.
+		 */
+		for (j = 0; j < 10; j++) {
+			usleep(200000);
+			fh = fopen(wins_hook_file, "r");
+			if (fh != NULL) {
+				break;
+			}
+		}
+
+		if (fh == NULL) {
+			if (errno == ENOENT) {
+				if (test_cases[i].should_succeed) {
+					torture_comment(
+						tctx,
+						"wins hook for '%s' failed\n",
+						test_cases[i].name);
+					ret = false;
+				}
+			} else {
+				torture_comment(
+					tctx,
+					"wins hook for '%s' unexpectedly failed with %d\n",
+					test_cases[i].name,
+					errno);
+				ret = false;
+			}
+		} else {
+			char readback[17] = {0};
+			size_t n = fread(readback, 1, 16, fh);
+			torture_comment(tctx,
+					"wins hook wrote '%s' read '%.*s'\n",
+					test_cases[i].name,
+					(int)n, readback);
+
+			if (! test_cases[i].should_succeed) {
+				torture_comment(tctx,
+						"wins hook for '%s' should fail\n",
+						test_cases[i].name);
+				ret = false;
+			}
+			fclose(fh);
+		}
+	}
+	err = unlink(wins_hook_file);
+	if (err != 0 && errno != ENOENT) {
+		torture_comment(tctx, "final unlink of '%s' failed\n",
+				wins_hook_file);
+	}
+	torture_assert(tctx, ret, "wins hook failure\n");
+	return ret;
+}
+
+
 /*
   test WINS operations
 */
 struct torture_suite *torture_nbt_wins(TALLOC_CTX *mem_ctx)
 {
-	struct torture_suite *suite = torture_suite_create(mem_ctx, "WINS");
+	struct torture_suite *suite = torture_suite_create(mem_ctx, "wins");
 
 	torture_suite_add_simple_test(suite, "wins", nbt_test_wins);
+	torture_suite_add_simple_test(suite, "wins_bad_names",
+				      nbt_test_wins_bad_names);
 
 	return suite;
 }
