@@ -2,12 +2,12 @@
 
 if [ $# -lt 1 ]; then
 	cat <<EOF
-Usage: dbcheck.sh PREFIX RELEASE
+Usage: dbcheck-oldrelease.sh PREFIX RELEASE
 EOF
 	exit 1
 fi
 
-PREFIX_ABS="$1"
+PREFIX="$1"
 RELEASE="$2"
 shift 2
 
@@ -108,13 +108,13 @@ fi
 
 undump()
 {
-	$samba_undump $release_dir $PREFIX_ABS/$RELEASE $samba_tdbrestore
+	$samba_undump $release_dir $PREFIX/$RELEASE $samba_tdbrestore
 }
 
 add_userparameters0()
 {
 	if [ x$RELEASE = x"release-4-1-0rc3" ]; then
-		$ldbmodify -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb <<EOF
+		$ldbmodify -H tdb://$PREFIX/${RELEASE}/private/sam.ldb <<EOF
 dn: cn=localdc,cn=domain controllers,dc=release-4-1-0rc3,dc=samba,dc=corp
 changetype: modify
 replace: userParameters
@@ -142,7 +142,7 @@ EOF
 add_userparameters1()
 {
 	if [ x$RELEASE = x"release-4-1-0rc3" ]; then
-		$ldbmodify -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb <<EOF
+		$ldbmodify -H tdb://$PREFIX/${RELEASE}/private/sam.ldb <<EOF
 dn: cn=administrator,cn=users,dc=release-4-1-0rc3,dc=samba,dc=corp
 changetype: modify
 replace: userParameters
@@ -170,7 +170,7 @@ EOF
 add_userparameters2()
 {
 	if [ x$RELEASE = x"release-4-1-0rc3" ]; then
-		$ldbmodify -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb <<EOF
+		$ldbmodify -H tdb://$PREFIX/${RELEASE}/private/sam.ldb <<EOF
 dn: cn=krbtgt,cn=users,dc=release-4-1-0rc3,dc=samba,dc=corp
 changetype: modify
 replace: userParameters
@@ -189,7 +189,7 @@ EOF
 add_userparameters3()
 {
 	if [ x$RELEASE = x"release-4-1-0rc3" ]; then
-		$ldbmodify -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb <<EOF
+		$ldbmodify -H tdb://$PREFIX/${RELEASE}/private/sam.ldb <<EOF
 dn: cn=guest,cn=users,dc=release-4-1-0rc3,dc=samba,dc=corp
 changetype: modify
 replace: userParameters
@@ -221,8 +221,8 @@ EOF
 check_expected_userparameters()
 {
 	if [ x$RELEASE = x"release-4-1-0rc3" ]; then
-		tmpldif=$PREFIX_ABS/$RELEASE/expected-userParameters-after-dbcheck.ldif.tmp
-		TZ=UTC $ldbsearch -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb userParameters=* --scope=sub -b DC=release-4-1-0rc3,DC=samba,DC=corp userParameters --sorted | grep -v \# >$tmpldif
+		tmpldif=$PREFIX/$RELEASE/expected-userParameters-after-dbcheck.ldif.tmp
+		TZ=UTC $ldbsearch -H tdb://$PREFIX/${RELEASE}/private/sam.ldb userParameters=* --scope=sub -b DC=release-4-1-0rc3,DC=samba,DC=corp userParameters --sorted | grep -v \# >$tmpldif
 		diff -u $tmpldif $release_dir/expected-userParameters-after-dbcheck.ldif
 		if [ "$?" != "0" ]; then
 			return 1
@@ -233,7 +233,7 @@ check_expected_userparameters()
 
 reindex()
 {
-	$PYTHON $BINDIR/samba-tool dbcheck --reindex -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb "$@"
+	$PYTHON $BINDIR/samba-tool dbcheck --reindex -H tdb://$PREFIX/${RELEASE}/private/sam.ldb "$@"
 }
 
 do_current_version_mod()
@@ -243,7 +243,7 @@ do_current_version_mod()
 		# changing the attribute with current Samba fixes it, and that
 		# a fixed attriute isn't unfixed by dbcheck.
 		tmpldif=$release_dir/sudoers2-mod.ldif
-		$ldbmodify -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb $tmpldif
+		$ldbmodify -H tdb://$PREFIX/${RELEASE}/private/sam.ldb $tmpldif
 	fi
 	return 0
 }
@@ -251,14 +251,14 @@ do_current_version_mod()
 check_expected_before_values()
 {
 	if [ x$RELEASE = x"release-4-1-0rc3" ]; then
-		tmpldif=$PREFIX_ABS/$RELEASE/expected-replpropertymetadata-before-dbcheck.ldif.tmp
-		TZ=UTC $ldbsearch -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb cn=ops_run_anything --scope=one -b OU=SUDOers,DC=release-4-1-0rc3,DC=samba,DC=corp \* replpropertymetadata --sorted --show-binary >$tmpldif
+		tmpldif=$PREFIX/$RELEASE/expected-replpropertymetadata-before-dbcheck.ldif.tmp
+		TZ=UTC $ldbsearch -H tdb://$PREFIX/${RELEASE}/private/sam.ldb cn=ops_run_anything --scope=one -b OU=SUDOers,DC=release-4-1-0rc3,DC=samba,DC=corp \* replpropertymetadata --sorted --show-binary >$tmpldif
 		diff -u $tmpldif $release_dir/expected-replpropertymetadata-before-dbcheck.ldif
 		if [ "$?" != "0" ]; then
 			return 1
 		fi
 
-		TZ=UTC $ldbsearch -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb cn=ops_run_anything2 --scope=one -b OU=SUDOers,DC=release-4-1-0rc3,DC=samba,DC=corp \* replpropertymetadata --sorted --show-binary | grep -v originating_change_time | grep -v whenChanged >$tmpldif
+		TZ=UTC $ldbsearch -H tdb://$PREFIX/${RELEASE}/private/sam.ldb cn=ops_run_anything2 --scope=one -b OU=SUDOers,DC=release-4-1-0rc3,DC=samba,DC=corp \* replpropertymetadata --sorted --show-binary | grep -v originating_change_time | grep -v whenChanged >$tmpldif
 
 		# Here we remove originating_change_time and whenChanged as
 		# these are time-dependent, caused by the ldbmodify above.
@@ -268,14 +268,14 @@ check_expected_before_values()
 			return 1
 		fi
 
-		TZ=UTC $ldbsearch -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb cn=ops_run_anything3 --scope=one -b OU=SUDOers,DC=release-4-1-0rc3,DC=samba,DC=corp \* replpropertymetadata --sorted --show-binary >$tmpldif
+		TZ=UTC $ldbsearch -H tdb://$PREFIX/${RELEASE}/private/sam.ldb cn=ops_run_anything3 --scope=one -b OU=SUDOers,DC=release-4-1-0rc3,DC=samba,DC=corp \* replpropertymetadata --sorted --show-binary >$tmpldif
 		diff -u $tmpldif $release_dir/expected-replpropertymetadata-before-dbcheck3.ldif
 		if [ "$?" != "0" ]; then
 			return 1
 		fi
 	elif [ x$RELEASE = x"release-4-5-0-pre1" ]; then
-		tmpldif=$PREFIX_ABS/$RELEASE/rootdse-version.initial.txt.tmp
-		TZ=UTC $ldbsearch -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb --scope=base -b '' | grep highestCommittedUSN >$tmpldif
+		tmpldif=$PREFIX/$RELEASE/rootdse-version.initial.txt.tmp
+		TZ=UTC $ldbsearch -H tdb://$PREFIX/${RELEASE}/private/sam.ldb --scope=base -b '' | grep highestCommittedUSN >$tmpldif
 		diff -u $tmpldif $release_dir/rootdse-version.initial.txt
 		if [ "$?" != "0" ]; then
 			return 1
@@ -288,7 +288,7 @@ check_expected_before_values()
 dbcheck_objectclass()
 {
 	if [ x$RELEASE = x"release-4-1-6-partial-object" ]; then
-		$PYTHON $BINDIR/samba-tool dbcheck --selftest-check-expired-tombstones --cross-ncs --fix --yes -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb --attrs=objectclass "$@"
+		$PYTHON $BINDIR/samba-tool dbcheck --selftest-check-expired-tombstones --cross-ncs --fix --yes -H tdb://$PREFIX/${RELEASE}/private/sam.ldb --attrs=objectclass "$@"
 	else
 		return 1
 	fi
@@ -298,9 +298,9 @@ dbcheck_objectclass()
 dbcheck_deleted_objects()
 {
 	if [ x$RELEASE = x"alpha13" ]; then
-		basedn=$($ldbsearch -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb --scope base -b "" defaultNamingContext | grep -i defaultNamingContext | cut -d\  -f 2)
+		basedn=$($ldbsearch -H tdb://$PREFIX/${RELEASE}/private/sam.ldb --scope base -b "" defaultNamingContext | grep -i defaultNamingContext | cut -d\  -f 2)
 
-		$PYTHON $BINDIR/samba-tool dbcheck -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb "cn=deleted objects,$basedn" --scope base "$@"
+		$PYTHON $BINDIR/samba-tool dbcheck -H tdb://$PREFIX/${RELEASE}/private/sam.ldb "cn=deleted objects,$basedn" --scope base "$@"
 	else
 		return 1
 	fi
@@ -309,45 +309,45 @@ dbcheck_deleted_objects()
 # This should 'fail', because it returns the number of modified records
 dbcheck()
 {
-	$PYTHON $BINDIR/samba-tool dbcheck --selftest-check-expired-tombstones --cross-ncs --fix --yes -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb "$@"
+	$PYTHON $BINDIR/samba-tool dbcheck --selftest-check-expired-tombstones --cross-ncs --fix --yes -H tdb://$PREFIX/${RELEASE}/private/sam.ldb "$@"
 }
 
 check_expected_after_values()
 {
 	if [ x$RELEASE = x"release-4-1-0rc3" ]; then
-		tmpldif=$PREFIX_ABS/$RELEASE/expected-replpropertymetadata-after-dbcheck.ldif.tmp
-		TZ=UTC $ldbsearch -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb cn=ops_run_anything --scope=one -b OU=SUDOers,DC=release-4-1-0rc3,DC=samba,DC=corp \* replpropertymetadata --sorted --show-binary >$tmpldif
+		tmpldif=$PREFIX/$RELEASE/expected-replpropertymetadata-after-dbcheck.ldif.tmp
+		TZ=UTC $ldbsearch -H tdb://$PREFIX/${RELEASE}/private/sam.ldb cn=ops_run_anything --scope=one -b OU=SUDOers,DC=release-4-1-0rc3,DC=samba,DC=corp \* replpropertymetadata --sorted --show-binary >$tmpldif
 		diff -u $tmpldif $release_dir/expected-replpropertymetadata-after-dbcheck.ldif
 		if [ "$?" != "0" ]; then
 			return 1
 		fi
-		TZ=UTC $ldbsearch -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb cn=ops_run_anything2 --scope=one -b OU=SUDOers,DC=release-4-1-0rc3,DC=samba,DC=corp \* replpropertymetadata --sorted --show-binary | grep -v originating_change_time | grep -v whenChanged >$tmpldif
+		TZ=UTC $ldbsearch -H tdb://$PREFIX/${RELEASE}/private/sam.ldb cn=ops_run_anything2 --scope=one -b OU=SUDOers,DC=release-4-1-0rc3,DC=samba,DC=corp \* replpropertymetadata --sorted --show-binary | grep -v originating_change_time | grep -v whenChanged >$tmpldif
 		diff -u $tmpldif $release_dir/expected-replpropertymetadata-after-dbcheck2.ldif
 		if [ "$?" != "0" ]; then
 			return 1
 		fi
-		TZ=UTC $ldbsearch -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb cn=ops_run_anything3 --scope=one -b OU=SUDOers,DC=release-4-1-0rc3,DC=samba,DC=corp \* replpropertymetadata --sorted --show-binary >$tmpldif
+		TZ=UTC $ldbsearch -H tdb://$PREFIX/${RELEASE}/private/sam.ldb cn=ops_run_anything3 --scope=one -b OU=SUDOers,DC=release-4-1-0rc3,DC=samba,DC=corp \* replpropertymetadata --sorted --show-binary >$tmpldif
 		diff -u $tmpldif $release_dir/expected-replpropertymetadata-after-dbcheck3.ldif
 		if [ "$?" != "0" ]; then
 			return 1
 		fi
 		# Check DomainDNS partition for replica locations
-		tmpldif=$PREFIX_ABS/$RELEASE/expected-replica-locations-after-dbcheck.ldif.tmp
-		$ldbsearch -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb cn=49a69498-9a85-48af-9be4-aa0b3e0054f9 --scope=one -b CN=Partitions,CN=Configuration,DC=release-4-1-0rc3,DC=samba,DC=corp msDS-NC-Replica-Locations >$tmpldif
+		tmpldif=$PREFIX/$RELEASE/expected-replica-locations-after-dbcheck.ldif.tmp
+		$ldbsearch -H tdb://$PREFIX/${RELEASE}/private/sam.ldb cn=49a69498-9a85-48af-9be4-aa0b3e0054f9 --scope=one -b CN=Partitions,CN=Configuration,DC=release-4-1-0rc3,DC=samba,DC=corp msDS-NC-Replica-Locations >$tmpldif
 		diff -u $tmpldif $release_dir/expected-replica-locations-after-dbcheck.ldif
 		if [ "$?" != "0" ]; then
 			return 1
 		fi
 		# Check ForestDNS partition for replica locations
-		$ldbsearch -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb cn=7d2a15af-c0d4-487c-847e-e036292bcc65 --scope=one -b CN=Partitions,CN=Configuration,DC=release-4-1-0rc3,DC=samba,DC=corp msDS-NC-Replica-Locations >$tmpldif
+		$ldbsearch -H tdb://$PREFIX/${RELEASE}/private/sam.ldb cn=7d2a15af-c0d4-487c-847e-e036292bcc65 --scope=one -b CN=Partitions,CN=Configuration,DC=release-4-1-0rc3,DC=samba,DC=corp msDS-NC-Replica-Locations >$tmpldif
 		diff -u $tmpldif $release_dir/expected-replica-locations-after-dbcheck2.ldif
 		if [ "$?" != "0" ]; then
 			return 1
 		fi
 	elif [ x$RELEASE = x"release-4-5-0-pre1" ]; then
 		echo $RELEASE checking after values
-		tmpldif=$PREFIX_ABS/$RELEASE/expected-links-after-dbcheck.ldif.tmp
-		$ldbsearch -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb --show-recycled --show-deleted --show-deactivated-link --reveal member memberOf lastKnownParent objectCategory lastKnownParent wellKnownObjects legacyExchangeDN sAMAccountType uSNChanged --sorted >$tmpldif
+		tmpldif=$PREFIX/$RELEASE/expected-links-after-dbcheck.ldif.tmp
+		$ldbsearch -H tdb://$PREFIX/${RELEASE}/private/sam.ldb --show-recycled --show-deleted --show-deactivated-link --reveal member memberOf lastKnownParent objectCategory lastKnownParent wellKnownObjects legacyExchangeDN sAMAccountType uSNChanged --sorted >$tmpldif
 		diff -u $tmpldif $release_dir/expected-links-after-dbcheck.ldif
 		if [ "$?" != "0" ]; then
 			return 1
@@ -355,8 +355,8 @@ check_expected_after_values()
 
 		# If in the future dbcheck has to make a change recorded in replPropertyMetadata,
 		# this test will fail and can be removed.
-		tmpversion=$PREFIX_ABS/$RELEASE/rootdse-version.final.txt.tmp
-		TZ=UTC $ldbsearch -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb --scope=base -b '' | grep highestCommittedUSN >$tmpversion
+		tmpversion=$PREFIX/$RELEASE/rootdse-version.final.txt.tmp
+		TZ=UTC $ldbsearch -H tdb://$PREFIX/${RELEASE}/private/sam.ldb --scope=base -b '' | grep highestCommittedUSN >$tmpversion
 		diff -u $tmpversion $release_dir/rootdse-version.final.txt
 		if [ "$?" != "0" ]; then
 			return 1
@@ -369,7 +369,7 @@ check_forced_duplicate_values()
 {
 	if [ x$RELEASE = x"release-4-1-0rc3" ]; then
 		ldif=$release_dir/forced-duplicate-value-for-dbcheck.ldif
-		TZ=UTC $ldbmodify -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb.d/DC%3DRELEASE-4-1-0RC3,DC%3DSAMBA,DC%3DCORP.ldb $ldif
+		TZ=UTC $ldbmodify -H tdb://$PREFIX/${RELEASE}/private/sam.ldb.d/DC%3DRELEASE-4-1-0RC3,DC%3DSAMBA,DC%3DCORP.ldb $ldif
 		if [ "$?" != "0" ]; then
 			return 1
 		fi
@@ -382,7 +382,7 @@ check_forced_duplicate_values()
 dbcheck_after_dup()
 {
 	if [ x$RELEASE = x"release-4-1-0rc3" ]; then
-		$PYTHON $BINDIR/samba-tool dbcheck --selftest-check-expired-tombstones --fix --yes -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb cn=administrator,cn=users,DC=release-4-1-0rc3,DC=samba,DC=corp "$@"
+		$PYTHON $BINDIR/samba-tool dbcheck --selftest-check-expired-tombstones --fix --yes -H tdb://$PREFIX/${RELEASE}/private/sam.ldb cn=administrator,cn=users,DC=release-4-1-0rc3,DC=samba,DC=corp "$@"
 	else
 		return 1
 	fi
@@ -391,8 +391,8 @@ dbcheck_after_dup()
 check_expected_after_dup_values()
 {
 	if [ x$RELEASE = x"release-4-1-0rc3" ]; then
-		tmpldif=$PREFIX_ABS/$RELEASE/expected-otherphone-after-dbcheck.ldif.tmp
-		TZ=UTC $ldbsearch -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb cn=administrator --scope=base -b cn=administrator,cn=users,DC=release-4-1-0rc3,DC=samba,DC=corp otherHomePhone --sorted --show-binary | grep -v \# | sort >$tmpldif
+		tmpldif=$PREFIX/$RELEASE/expected-otherphone-after-dbcheck.ldif.tmp
+		TZ=UTC $ldbsearch -H tdb://$PREFIX/${RELEASE}/private/sam.ldb cn=administrator --scope=base -b cn=administrator,cn=users,DC=release-4-1-0rc3,DC=samba,DC=corp otherHomePhone --sorted --show-binary | grep -v \# | sort >$tmpldif
 		diff -u $tmpldif $release_dir/expected-otherphone-after-dbcheck.ldif
 		if [ "$?" != "0" ]; then
 			return 1
@@ -404,21 +404,21 @@ check_expected_after_dup_values()
 # But having fixed it all up, this should pass
 dbcheck_clean()
 {
-	$PYTHON $BINDIR/samba-tool dbcheck --cross-ncs -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb "$@"
+	$PYTHON $BINDIR/samba-tool dbcheck --cross-ncs -H tdb://$PREFIX/${RELEASE}/private/sam.ldb "$@"
 }
 
 # This should 'fail', because it returns the number of modified records.
 # We need to run this against all versions without adprep 2016 (before Samba 4.19)
 dbcheck_acl_reset()
 {
-	$PYTHON $BINDIR/samba-tool dbcheck --reset-well-known-acls --cross-ncs --fix --yes -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb "$@"
+	$PYTHON $BINDIR/samba-tool dbcheck --reset-well-known-acls --cross-ncs --fix --yes -H tdb://$PREFIX/${RELEASE}/private/sam.ldb "$@"
 }
 # But having fixed it all up, this should pass.
 # We don't need to run this against 4.1.0rc3
 dbcheck_acl_reset_clean()
 {
 	if [ x$RELEASE != x"release-4-1-0rc3" ]; then
-		$PYTHON $BINDIR/samba-tool dbcheck --reset-well-known-acls --cross-ncs -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb "$@"
+		$PYTHON $BINDIR/samba-tool dbcheck --reset-well-known-acls --cross-ncs -H tdb://$PREFIX/${RELEASE}/private/sam.ldb "$@"
 	fi
 }
 
@@ -426,7 +426,7 @@ dbcheck_acl_reset_clean()
 dbcheck2()
 {
 	if [ x$RELEASE = x"release-4-1-0rc3" ]; then
-		$PYTHON $BINDIR/samba-tool dbcheck --selftest-check-expired-tombstones --cross-ncs --fix --yes -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb "$@"
+		$PYTHON $BINDIR/samba-tool dbcheck --selftest-check-expired-tombstones --cross-ncs --fix --yes -H tdb://$PREFIX/${RELEASE}/private/sam.ldb "$@"
 	else
 		exit 1
 	fi
@@ -435,14 +435,14 @@ dbcheck2()
 dbcheck_clean2()
 {
 	if [ x$RELEASE = x"release-4-1-0rc3" ]; then
-		$PYTHON $BINDIR/samba-tool dbcheck --selftest-check-expired-tombstones --cross-ncs -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb "$@"
+		$PYTHON $BINDIR/samba-tool dbcheck --selftest-check-expired-tombstones --cross-ncs -H tdb://$PREFIX/${RELEASE}/private/sam.ldb "$@"
 	fi
 }
 
 rm_deleted_objects()
 {
 	if [ x$RELEASE = x"release-4-1-0rc3" ]; then
-		TZ=UTC $ldbdel -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb.d/DC%3DRELEASE-4-1-0RC3,DC%3DSAMBA,DC%3DCORP.ldb 'CN=Deleted Objects,DC=RELEASE-4-1-0RC3,DC=SAMBA,DC=CORP'
+		TZ=UTC $ldbdel -H tdb://$PREFIX/${RELEASE}/private/sam.ldb.d/DC%3DRELEASE-4-1-0RC3,DC%3DSAMBA,DC%3DCORP.ldb 'CN=Deleted Objects,DC=RELEASE-4-1-0RC3,DC=SAMBA,DC=CORP'
 		if [ "$?" != "0" ]; then
 			return 1
 		fi
@@ -454,7 +454,7 @@ rm_deleted_objects()
 dbcheck3()
 {
 	if [ x$RELEASE = x"release-4-1-0rc3" ]; then
-		$PYTHON $BINDIR/samba-tool dbcheck --selftest-check-expired-tombstones --cross-ncs --fix --yes -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb "$@"
+		$PYTHON $BINDIR/samba-tool dbcheck --selftest-check-expired-tombstones --cross-ncs --fix --yes -H tdb://$PREFIX/${RELEASE}/private/sam.ldb "$@"
 	else
 		exit 1
 	fi
@@ -463,15 +463,15 @@ dbcheck3()
 dbcheck_clean3()
 {
 	if [ x$RELEASE = x"release-4-1-0rc3" ]; then
-		$PYTHON $BINDIR/samba-tool dbcheck --selftest-check-expired-tombstones --cross-ncs -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb "$@"
+		$PYTHON $BINDIR/samba-tool dbcheck --selftest-check-expired-tombstones --cross-ncs -H tdb://$PREFIX/${RELEASE}/private/sam.ldb "$@"
 	fi
 }
 
 check_expected_after_deleted_objects()
 {
 	if [ x$RELEASE = x"release-4-1-0rc3" ]; then
-		tmpldif=$PREFIX_ABS/$RELEASE/expected-deleted_objects-after-dbcheck.ldif.tmp
-		TZ=UTC $ldbsearch -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb cn=deleted\ objects --scope=base -b cn=deleted\ objects,DC=release-4-1-0rc3,DC=samba,DC=corp objectClass description isDeleted isCriticalSystemObject objectGUID showInAdvancedViewOnly systemFlags --sorted --show-binary --show-deleted | grep -v \# | sort >$tmpldif
+		tmpldif=$PREFIX/$RELEASE/expected-deleted_objects-after-dbcheck.ldif.tmp
+		TZ=UTC $ldbsearch -H tdb://$PREFIX/${RELEASE}/private/sam.ldb cn=deleted\ objects --scope=base -b cn=deleted\ objects,DC=release-4-1-0rc3,DC=samba,DC=corp objectClass description isDeleted isCriticalSystemObject objectGUID showInAdvancedViewOnly systemFlags --sorted --show-binary --show-deleted | grep -v \# | sort >$tmpldif
 		diff -u $tmpldif $release_dir/expected-deleted_objects-after-dbcheck.ldif
 		if [ "$?" != "0" ]; then
 			return 1
@@ -483,28 +483,28 @@ check_expected_after_deleted_objects()
 referenceprovision()
 {
 	if [ x$RELEASE = x"release-4-0-0" ]; then
-		$PYTHON $BINDIR/samba-tool domain provision --server-role="dc" --domain=SAMBA --host-name=ares --realm=${RELEASE}.samba.corp --targetdir=$PREFIX_ABS/${RELEASE}_reference --use-ntvfs --host-ip=127.0.0.1 --host-ip6=::1 --function-level=2003 --base-schema=2008_R2_old
+		$PYTHON $BINDIR/samba-tool domain provision --server-role="dc" --domain=SAMBA --host-name=ares --realm=${RELEASE}.samba.corp --targetdir=$PREFIX/${RELEASE}_reference --use-ntvfs --host-ip=127.0.0.1 --host-ip6=::1 --function-level=2003 --base-schema=2008_R2_old
 
 		# on top of this, also apply 2008R2 changes we accidentally missed in the past
-		$PYTHON $BINDIR/samba-tool domain schemaupgrade -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb --ldf-file=samba-4.7-missing-for-schema45.ldif,fix-forest-rev.ldf
+		$PYTHON $BINDIR/samba-tool domain schemaupgrade -H tdb://$PREFIX/${RELEASE}/private/sam.ldb --ldf-file=samba-4.7-missing-for-schema45.ldif,fix-forest-rev.ldf
 	fi
 }
 
 ldapcmp()
 {
 	if [ x$RELEASE = x"release-4-0-0" ]; then
-		$PYTHON $BINDIR/samba-tool ldapcmp tdb://$PREFIX_ABS/${RELEASE}_reference/private/sam.ldb tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb --two --skip-missing-dn --filter=dnsRecord,displayName,msDS-SupportedEncryptionTypes,servicePrincipalName
+		$PYTHON $BINDIR/samba-tool ldapcmp tdb://$PREFIX/${RELEASE}_reference/private/sam.ldb tdb://$PREFIX/${RELEASE}/private/sam.ldb --two --skip-missing-dn --filter=dnsRecord,displayName,msDS-SupportedEncryptionTypes,servicePrincipalName
 	fi
 }
 
 ldapcmp_sd()
 {
 	if [ x$RELEASE = x"release-4-0-0" ]; then
-		$PYTHON $BINDIR/samba-tool ldapcmp tdb://$PREFIX_ABS/${RELEASE}_reference/private/sam.ldb tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb --two --sd --skip-missing-dn --filter=servicePrincipalName
+		$PYTHON $BINDIR/samba-tool ldapcmp tdb://$PREFIX/${RELEASE}_reference/private/sam.ldb tdb://$PREFIX/${RELEASE}/private/sam.ldb --two --sd --skip-missing-dn --filter=servicePrincipalName
 	fi
 }
 
-remove_directory $PREFIX_ABS/${RELEASE}_reference
+remove_directory $PREFIX/${RELEASE}_reference
 
 testit $RELEASE undump || failed=$(expr $failed + 1)
 testit "reindex" reindex || failed=$(expr $failed + 1)
@@ -538,10 +538,10 @@ testit "referenceprovision" referenceprovision || failed=$(expr $failed + 1)
 testit "ldapcmp" ldapcmp || failed=$(expr $failed + 1)
 testit "ldapcmp_sd" ldapcmp_sd || failed=$(expr $failed + 1)
 
-if [ -d $PREFIX_ABS/${RELEASE} ]; then
-	rm -fr $PREFIX_ABS/${RELEASE}
+if [ -d $PREFIX/${RELEASE} ]; then
+	rm -fr $PREFIX/${RELEASE}
 fi
 
-remove_directory $PREFIX_ABS/${RELEASE}_reference
+remove_directory $PREFIX/${RELEASE}_reference
 
 exit $failed
